/* This library allows to contraol an analog input as a simple button. It also
 * handles a threshold to make it more feasable.
 * Author:
 * Efrain Calderon
 * E- mail: darkcerano@gmial.com
 */
#include "WProgram.h"
#include "Pulse.h"

/*
 * two-wire constructor.
 * Sets which wires should control the motor.
 */
Pulse::Pulse(int pin, unsigned long ms)
{
  this->init_time = millis();      // time stamp in ms since the pin will be active
  this->inverted = false;          // this is an inverted pulse (LOW active)
  this->active = false;            // current state of the pin
  this->width_ms = ms;             // amount of time that the pin will be active
  this->n_pulses = 0;              // number of pulses left
  this->wait = false;              // wait flag
  this->wait_time = 10;            // wait this time to start the next pulse
  
  // Arduino pin used:
  this->pulse_pin = pin;

  // setup the pins on the microcontroller:
  pinMode(this->pulse_pin, OUTPUT);
  digitalWrite(pin, LOW);
  
}

/*
 * two-wire constructor.
 * Sets which wires should control the motor.
 */
Pulse::Pulse(int pin, unsigned long ms, boolean inverted)
{
  this->init_time = millis();      // time stamp in ms since the pin will be active
  this->inverted = inverted;       // this is an inverted pulse (LOW active)
  this->active = false;            // current state of the pin
  this->width_ms = ms;             // amount of time that the pin will be active
  this->n_pulses = 0;              // number of pulses left
  this->wait = false;              // wait flag
  this->wait_time = 10;           // wait this time to start the next pulse
  
  // Arduino pins for the motor control connection:
  this->pulse_pin = pin;

  // setup the pins on the microcontroller:
  pinMode(this->pulse_pin, OUTPUT);
  digitalWrite(pin, HIGH);
  
}

// Allow to set the pulse width
void Pulse::setWidth(unsigned long ms)
{
  this->width_ms = ms;
}

/*
  Generates a single pulse
 */
void Pulse::trigger(void)
{  
  if (!(this->active)) {
      this->init_time = millis();
	  this->active = true;
	  startPulse();
	  this->n_pulses = 1;
   }
}

/* 
  Generates a train pulse.
  n  Number of pulses
*/
void Pulse::trigger(int n)
{  
  if (!(this->active)) {
      startPulse();
      this->active = true;
	  this->n_pulses = n;
   }
}

/* Ininitates the pulse generation*/
void Pulse::startPulse()
{  
 
   this->init_time = millis();
   if (this->inverted) {
	  // get the timeStamp of when you stepped:
      digitalWrite(this->pulse_pin, LOW);
   }
   else{
	  digitalWrite(this->pulse_pin, HIGH);
   }
   Serial.print("pressed pin ");
   Serial.println(this->pulse_pin);
}

/*
  This functions MUST be called periodically
 */
void Pulse::polling(void)
{  
    
  if (this->active) {
	  // does it have to wait to triggger the next pulse or not?
	  if(!this->wait)
	  {
	     // move only if the appropriate delay has passed:
	     if (millis() - this->init_time >= this->width_ms){
		    // get the timeStamp of when you stepped:
		    digitalWrite(this->pulse_pin, !(digitalRead(this->pulse_pin)));
		    this->wait = true;
			this->n_pulses --;
		 }
	  }
	  else { //if it does have to wait
	     if (millis() - this->init_time >= this->wait_time){
		    // get the timeStamp of when you stepped:
		    startPulse();
		    this->wait = false;
		 }
	  }
	  if(this->n_pulses == 0)
	  {
		this->active = false;
	  }
  }
}

void Pulse::toggle(void)
{
  this->inverted = !(this->inverted);
}
/*
  version() returns the version of the library:
*/
int Pulse::version(void)
{
  return 1;
}
